import requests


class BaseApi:
    def __init__(self,timeout=30):
        self.session = requests.session()
        self.timeout = timeout

    def common_get_request(self,url,header):
        r=requests.get(url,headers=header,timeout=self.timeout)
        result=r.json()
        if result['code'] != 200:
            print('Error return{} when try to use get function{}'.format(result,url))
        else:
            return result

    def common_post_request(self,url,header,params):
        r=requests.post(url,headers=header,data=params,timeout=self.timeout)
        result=r.json()
        if result['code'] != 200:
            print('Error return {} when try to post=> {}'.format(result,params,url))
        else:
            return result




import argparse

from .net.kugou_api import KugouCloud
from .net.netease_api import NetEaseCloud
from .net.oneting_api import OneCloud
from .net.qq_api import QQMusic
from .net.xiami_api import XiaMiCloud
from .utils.downloader import download_from_url

desc="Search && Download Music cli"
version_info="""
 ______     _____     __    __     __  __     ______     __     ______    
/\  ___\   /\  __-.  /\ "-./  \   /\ \/\ \   /\  ___\   /\ \   /\  ___\   
\ \___  \  \ \ \/\ \ \ \ \-./\ \  \ \ \_\ \  \ \___  \  \ \ \  \ \ \____  
 \/\_____\  \ \____-  \ \_\ \ \_\  \ \_____\  \/\_____\  \ \_\  \ \_____\ 
  \/_____/   \/____/   \/_/  \/_/   \/_____/   \/_____/   \/_/   \/_____/
  
Search && Download Music Cli
version 0.01a 
"""




def search_or_download(music_name,offset,platfrom='netease',choose=True,index=1,output='./'):
    if platfrom == 'netease':
        net_api=NetEaseCloud()
        if choose:
            net_api.show_music_info(music_name,offset)
        else:
            mIds=net_api.get_music_ids(music_name,offset)
            id=mIds[index]
            neturl=net_api.get_music_url(id)
            download_from_url(neturl,output+music_name+'.mp3')
    elif platfrom == 'qq':
        qq_api=QQMusic()
        if choose:
            qq_api.show_music_infos(music_name,offset)
        else:
            qqurl=qq_api.get_music_url(music_name,index,offset)
            download_from_url(qqurl,output+music_name+'.m4a')
        pass
    elif platfrom == '1ting':
        oneting_api=OneCloud()
        if choose:
            oneting_api.show_music_infos(music_name,offset)
        else:
            onetingurl=oneting_api.get_music_url(music_name,offset,index)
            print(onetingurl)
            download_from_url(onetingurl,output+music_name+'.mp3')
        pass
    elif platfrom == 'xiami':
        xiami_api=XiaMiCloud()
        if choose:
            xiami_api.show_music_infos(music_name,offset)
        else:
            url=xiami_api.get_music_url(music_name,offset,index)
            download_from_url(url,output+music_name+'.mp3')
        pass
    elif platfrom == 'kugou':
        kugou_api=KugouCloud()
        if choose:
            kugou_api.show_music_infos(music_name,offset)
        else:
            url=kugou_api.get_music_url(music_name,offset,index)
            download_from_url(url,output+music_name+'.mp3')
        pass


def main():
    music_platform = ['netease', 'qq', '1ting', 'xiami', 'kugou']
    parse = argparse.ArgumentParser(description=desc)
    parseGroup = parse.add_argument_group()
    parseGroup.add_argument("-s", "--search", action="store_true", help="search mode")
    parseGroup.add_argument("-d", "--download", action="store_true", help="download mode")
    parse.add_argument("-v", "--version", action="store_true", help="print product version")
    parse.add_argument("-n", "--name", type=str, help="input music name")
    parse.add_argument("-i", "--index", type=int, default=0, help="index for download music")
    parse.add_argument("-p", "--platform", type=str, choices=music_platform, default="netease",
                       help="platform you want to download include:qq,1ting,xiammi,kugou")
    parse.add_argument("-o", "--output", type=str, default="./", help="path you music")
    parse.add_argument("-g", "--page", type=int, default=1, help="the page you want to change")
    args = parse.parse_args()

    name = args.name
    platform = args.platform
    offset = args.page
    index = args.index
    output = args.output
    if args.version:
        print(version_info)
    if args.search:
        print("search\tMusicName:" + args.name + "\tPlatform:" + args.platform)
        search_or_download(name, offset, platform)
    elif args.download:
        search_or_download(name, offset, platform, False, index - 1, output)


if __name__=="__main__":
    main()



import requests

from ..net.base_api import BaseApi
from ..constants.oneting_constants import get_music_search_url, oneting_headers, oneting_base_download_url
from ..utils.shower import show_title, show_music


class OneCloud(BaseApi):

    def __init__(self,timeout=30):
        BaseApi.__init__(BaseApi(),timeout)
        self.timeout=timeout

    def get_request(self,url):
        r = requests.get(url,timeout=self.timeout)
        result = r.json()
        return result
    def get_music_info(self,music_name,page_num):
        page_num-=1
        url = get_music_search_url(music_name, page_num)
        r = self.get_request(url)
        song_file_paths = r['results']
        return song_file_paths

    def show_music_infos(self,music_name,page_mun):
        song_file_paths=self.get_music_info(music_name,page_mun)
        show_title()
        i = 1
        for song_files in song_file_paths:
            author = song_files['singer_name']
            show_music(i,music_name,author)
            i += 1

    def get_music_url(self,music_name,page_num,index):
        song_file_paths = self.get_music_info(music_name, page_num)
        if len(song_file_paths)>=index:
            song_file=song_file_paths[index]
            download_url=oneting_base_download_url+song_file['song_filepath']
            download_url=download_url.replace('wma','mp3')
            return download_url
        else:
            print("索引超出范围")





# -*- coding: utf-8 -*-

import os
import base64
import json
import binascii
from Crypto.Cipher import AES

from ..constants.encrypt_constants import modulus, nonce, pub_key


def encrypted_request(text):
    text = json.dumps(text)
    sec_key = create_secret_key(16)
    enc_text = aes_encrypt(aes_encrypt(text, nonce), sec_key)
    enc_sec_key = rsa_encrypt(sec_key, pub_key, modulus)
    data = {'params': enc_text, 'encSecKey': enc_sec_key}
    return data


def aes_encrypt(text, sec_key):
    pad = 16 - len(text) % 16
    text = text + chr(pad) * pad
    encryptor = AES.new(sec_key, 2, '0102030405060708')
    cipher_text = encryptor.encrypt(text)
    cipher_text = base64.b64encode(cipher_text).decode('utf-8')
    return cipher_text


def rsa_encrypt(text, public_key, p_modulus):
    text = text[::-1]
    rs = pow(int(binascii.hexlify(text), 16), int(public_key, 16), int(p_modulus, 16))
    return format(rs, 'x').zfill(256)


def create_secret_key(size):
    return binascii.hexlify(os.urandom(size))[:16]



import requests
from ..constants.qqmusic_constants import qq_headers, qq_get_music_info_url, qq_download_base_url
from ..net.base_api import BaseApi
from ..utils.shower import show_music, show_title


class QQMusic(BaseApi):
    def __init__(self, timeout=30):
        BaseApi.__init__(BaseApi(), timeout)
        self.timeout = timeout

    def get_request(self,url,header):
        r=requests.get(url,headers=header,timeout=self.timeout)
        result=r.json()
        if result['code'] != 0:
            print('Error return{} when try to use get function{}'.format(result,url))
        else:
            return result

    def get_music_info(self,music_name,page_num):
        music_search_url = qq_get_music_info_url(music_name, page_num)
        r = self.get_request(music_search_url, qq_headers)
        myDatas = r['data']['song']['list']
        return myDatas

    def show_music_infos(self,music_name,page_num=1):
        myDatas=self.get_music_info(music_name,page_num)
        show_title()
        i=1
        for m in myDatas:
            authors = m['singer']
            author = ''
            for a in authors:
                author += a['name']
            show_music(i,music_name,author)
            i += 1


    def get_music_url(self,music_name,index,page_num=1):
        myDatas = self.get_music_info(music_name, page_num)
        data=myDatas[index]
        if 'songmid' in data and 'songid' in data:
            mymId = data['songmid']
            myId=data['songid']
            qq_download_url = qq_download_base_url + '?songmid=' + mymId + '&format=json'
            r=self.get_request(qq_download_url,header=qq_headers)
            if 'url' in r:
                download_url = r['url'][str(myId)]
                return "http://"+download_url
            else:
                print("未知错误")
        else:
            print("超出边界")



import os
from urllib.request import urlopen
from tqdm import tqdm
import requests
# 下载工具
def download_from_url(url, dst):
    """
    @param: url to download file
    @param: dst place to put the file
    """
    file_size = int(urlopen(url).info().get('Content-Length', -1))
    if os.path.exists(dst):
        first_byte = os.path.getsize(dst)
    else:
        first_byte = 0
    if first_byte >= file_size:
        return file_size
    header = {"Range": "bytes=%s-%s" % (first_byte, file_size)}
    pbar = tqdm(
        total=file_size, initial=first_byte,
        unit='B', unit_scale=True, desc=url.split('/')[-1])
    req = requests.get(url, headers=header, stream=True)
    with(open(dst, 'ab')) as f:
        for chunk in req.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
                pbar.update(1024)
    pbar.close()
    return file_size



qq_search_url_first='https://c.y.qq.com/soso/fcgi-bin/search_for_qq_cp?w='
qq_search_url_index='&p='
qq_search_url_last='&n=10&format=json'

qq_download_base_url='https://c.y.qq.com/v8/fcg-bin/fcg_play_single_song.fcg'

qq_headers={
'referer': 'https://m.y.qq.com/#search',
'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1'
}

def qq_get_music_info_url(music_name,page_num):
    return qq_search_url_first+music_name+qq_search_url_index+str(page_num)+qq_search_url_last




xiami_search_url_first='http://api.xiami.com/web?key='
xiami_search_url_index='&v=2.0&app_key=1&r=search/songs&page='
xiami_search_url_last='&limit=10'
xiami_header = {'Referer': 'http://m.xiami.com/',
                'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1'}

def get_search_url(music_name,page_num):
    return xiami_search_url_first+music_name+xiami_search_url_index+str(page_num)+xiami_search_url_last




import requests

from ..constants.kugou_constants import get_search_url, kugou_header, kugou_base_download_url
from ..net.base_api import BaseApi
from ..utils.shower import show_music, show_out_of_bound, show_title


class KugouCloud(BaseApi):
    def __init__(self,timeout=30):
        BaseApi.__init__(BaseApi(),timeout)
        self.timeout=timeout
    def get_request(self,url,header):
        r=requests.get(url,headers=header,timeout=self.timeout)
        result=r.json()
        if result['status'] != 1:
            print('Error return{} when try to use get function{}'.format(result,url))
        else:
            return result
    def get_music_info(self,music_name,page_num):
        url=get_search_url(music_name,page_num)
        r=self.get_request(url,kugou_header)
        infos=r['data']['info']
        return infos

    def show_music_infos(self,music_name,page_num):
        infos=self.get_music_info(music_name,page_num)
        show_title()
        i=1
        for info in infos:
            authors=info['singername']
            music_name=info['songname_original']
            show_music(i,music_name,authors)
            i+=1

    def get_music_url(self,music_name,page_num,index):
        infos=self.get_music_info(music_name,page_num)
        if len(infos) >= index:
            info=infos[index]
            hash_code = info['hash']
            get_download_url=kugou_base_download_url+hash_code
            download_r=self.get_request(get_download_url,header=kugou_header)
            download_url=download_r['url']
            return download_url
        else:
            show_out_of_bound()






import requests

from ..constants.netease_constants import headers, get_song_url, netease_music_search_url,netease_song_download_url
from ..encrypt.netease_encrypt import encrypted_request
from ..net.base_api import BaseApi
from ..utils.shower import show_music, show_title


class NetEaseCloud(BaseApi):
    def __init__(self,timeout=30):
        BaseApi.__init__(BaseApi(),timeout)
        self.timeout=timeout


    def post_request(self,url,params):
        data =encrypted_request(params)
        r=requests.post(url,data=data,headers=headers,timeout=self.timeout)
        result=r.json()
        if result['code'] != 200:
            print('Error return {} when try to post {} => {}'.format(result,data,url))
        else:
            return result

    def get_song(self, song_id):
        """
        Get song info by song id
        :param song_id:
        :return:
        """
        url = get_song_url(song_id)
        result = self.common_get_request(url,headers)

        return result['songs'][0]
    def get_music_url(self, song_id, bit_rate=320000):
        """Get a song's download url.
        :params song_id: song id<int>.
        :params bit_rate: {'MD 128k': 128000, 'HD 320k': 320000}
        :return:
        """
        url = netease_song_download_url
        csrf = ''
        params = {'ids': [song_id], 'br': bit_rate, 'csrf_token': csrf}
        result = self.post_request(url, params)
        song_url = result['data'][0]['url']
        return song_url

    def get_music_id_json(self,music_name,offset):
        url=netease_music_search_url
        params={'s':music_name,'type':1,'offset':offset,'limit':10}
        result=self.common_post_request(url,headers,params)
        if 'songs' in result['result']:
            return result['result']['songs']


    def get_music_ids(self,music_name,offset=1):
        myIdJsons = self.get_music_id_json(music_name, offset)
        myIds=[]
        for myIdJson in myIdJsons:
            myId=myIdJson['id']
            myIds.append(myId)
        return myIds

    # 获取音乐的作者和MusicId
    def get_musics_info(self,music_name,offset=1):
        myIdJsons=self.get_music_id_json(music_name,offset)
        music_infos=[]
        index=1
        if myIdJsons != None:
            for myIdJson in myIdJsons:
                music_info = {}
                myId = myIdJson['id']
                myAuthors=myIdJson['ar']
                authors = ''
                for author in myAuthors:
                    authors += author['name']
                music_info['author']=authors
                music_info['id']=myId
                music_info['music_name']=music_name
                music_info['index']=index
                music_infos.append(music_info)
                index+=1
            return music_infos
        else:
            print("出现错误")
    def show_music_info(self,music_name,offset=1):
        music_infos=self.get_musics_info(music_name,offset)
        if music_infos != None:
            show_title()
            for music_info in music_infos:
                show_music(music_info['index'],music_info['music_name'],music_info['author'])
                # print(str(music_info['index'])+"\t"+music_info['music_name']+"\t"+music_info['author'])
        else:
            print("出现错误")

    # 获取音乐直链
    def get_music_download_url(self,selected_id):
        music=self.get_song(selected_id)
        song_id=music['id']
        music_url=self.get_music_url(song_id)
        return music_url


oneting_search_first='http://so.1ting.com/song/json?q='
oneting_search_index='&type=1&offset='
oneting_search_last='&limit=10'
oneting_base_download_url='http://96.1ting.com'
oneting_headers={
'referer': 'http://so.1ting.com/',
'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1'
}

def get_music_search_url(music_name,page_num):
    return oneting_search_first+music_name+oneting_search_index+str(page_num)+oneting_search_last





def show_music(index,music_name,authors):
    print(str(index) + "\t" + music_name + "\t" + authors)

def show_title():
    print("Index\tMusicName\tMusicAuthor")

def show_out_of_bound():
    print('越界错误')


# -*- coding:utf-8 -*-

# Encrypt key
modulus = '00e0b509f6259df8642dbc35662901477df22677ec152b5ff68ace615bb7b725152b3ab17a876aea8a5aa76d2e417629ec4ee341f56135fccf695280104e0312ecbda92557c93870114af6c9d05c4f7f0c3685b7a46bee255932575cce10b424d813cfe4875d3e82047b97ddef52741d546b8e289dc6935b3ece0462db0a22b8e7'
nonce = '0CoJUm6Qyw8W8jud'
pub_key = '010001'


headers={
    'Accept': '*/*',
    'Host': 'music.163.com',
    'User-Agent': 'curl/7.51.0',
    'Referer': 'http://music.163.com',
    'Cookie': 'appver=2.0.2'
}

netease_song_download_url = 'http://music.163.com/weapi/song/enhance/player/url?csrf_token='
netease_music_search_url='http://music.163.com/api/cloudsearch/pc'

def get_song_url(music_id):
    return 'http://music.163.com/api/song/detail/?ids=[{}]'.format(music_id)


def get_album_url(album_id):
    return 'http://music.163.com/api/album/{}/'.format(album_id)


def get_artist_url(artist_id):
    return 'http://music.163.com/api/artist/{}'.format(artist_id)


def get_playlist_url(playlist_id):
    return 'http://music.163.com/api/playlist/detail?id={}'.format(playlist_id)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
        name='sdmusic',
        version='0.02',
        packages=find_packages('src'),
        package_dir = {'':'src'},


    install_requires=[
                'requests>=2.17.3',
                'pycrypto>=2.6.1',
                'mutagen>=1.38.0',
                'tqdm>=4.19.5'
    ],

    entry_points={
        'console_scripts': [
                        'sdmusic = sd_music.start:main',
        ]
            
    },

        license='MIT',
        author='pcdack',
        author_email='ibackud@gmail.com',
        url='https://github.com/pcdack',
        description='Search && Download Music from netease qq,1ting,kugou and xiami',
        keywords=['sdmusic', 'netease','qq','kugou', 'music', 'downloader'],
    )



import requests

from ..constants.xiami_constants import get_search_url, xiami_header
from ..net.base_api import BaseApi
from ..utils.shower import show_music, show_out_of_bound


class XiaMiCloud(BaseApi):
    def __init__(self, timeout=30):
        BaseApi.__init__(BaseApi(), timeout)
        self.timeout = timeout

    def get_request(self,url,header):
        r=requests.get(url,headers=header,timeout=self.timeout)
        result=r.json()
        if result['state'] != 0:
            print('Error return{} when try to use get function{}'.format(result,url))
        else:
            return result

    def get_music_info(self,music_name,page_num):
        url = get_search_url(music_name, page_num)
        r = self.get_request(url, xiami_header)
        infos = r['data']['songs']
        return infos

    def show_music_infos(self,music_name,page_num):
        infos=self.get_music_info(music_name,page_num)
        i=1
        for info in infos:
            author = info['artist_name']
            show_music(i,music_name,author)
            i+=1

    def get_music_url(self,music_name,page_num,index):
        infos=self.get_music_info(music_name,page_num)
        if len(infos)>=index:
            info=infos[index]
            download_url = info['listen_file']
            return download_url
        else:
            show_out_of_bound()



kugou_search_url_first='http://mobilecdn.kugou.com/api/v3/search/song?keyword='
kugou_search_url_index='&format="json"&page='
kugou_search_url_last='&pagesize=10'
kugou_base_download_url='http://m.kugou.com/app/i/getSongInfo.php?cmd=playInfo&hash='
kugou_header = {'Referer': 'http://m.kugou.com/v2/static/html/search.html',
                     'User-Agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1'}


def get_search_url(music_name,page_num):
    return kugou_search_url_first+music_name+kugou_search_url_index+str(page_num)+kugou_search_url_last
